// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: ['slug'],
    emits: ['submit', 'cancel'],
    beforeCreate() {
      components.setParent(this)
    },

    // Listen on mounted event...
    mounted() {
      // logic
      if (this.slug) {
        this.ctx.processManager.fetchManager(this.slug)
          .then((manager) => {
            this.form = manager
            this.endpoint = new URL(this.form.options.url)
          })
      } else {
        this.form.service = this.ctx.uuidv4()
        this.form.options.slug = this.form.service
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        form: {
          service: '',
          options: {
            slug: '',
            url: '',
            type: 'process-manager',
            redirect: true,
            name: '',
            description: '',
            authenticator: 'standard',
          }
        },
        endpoint: {
          protocol: 'http:',
          hostname: '',
          port: 9018,
        },
        protocolOptions: [
          {
            label: 'HTTP',
            value: 'http:',
          },
          {
            label: 'HTTPS',
            value: 'https:',
          },
        ],
      }
    },

    // Define methods for the ui...
    methods: {
      submit() {
        this.form.options.url = `${this.endpoint.protocol}//${this.endpoint.hostname}:${this.endpoint.port}`
        if (!this.slug) {
          this.ctx.processManager.postManager(this.form.options)
            .then((res) => {
              this.$emit('submit', res, this.form)
            })    
        } else {
          this.ctx.processManager.putManager(this.form.options.slug, this.form.options)
            .then((res) => {
              this.$emit('submit', res, this.form)
            })      
        }
      },
      cancel() {
        this.$emit('cancel')
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.form().append(() => [
          components.caption('general'),
          components.input('ID', 'text', this.form.options.slug, (val) => this.form.options.slug = val).attrs({ readonly: true }),
          components.input('name', 'text', this.form.options.name, (val) => this.form.options.name = val).attrs({ required: true }),
          components.input('description', 'text', this.form.options.description, (val) => this.form.options.description = val),
          components.caption('URL'),
          components.input('protocol', 'select', this.endpoint.protocol, (val) => this.endpoint.protocol = val).attrs({ options: this.protocolOptions }),
          components.input('hostname', 'text', this.endpoint.hostname, (val) => this.endpoint.hostname = val).attrs({ required: true }),
          components.input('port', 'number', this.endpoint.port, (val) => this.endpoint.port = val).attrs({ required: true }),
        ]).on({
          onSubmit: () => this.submit(),
          onCancel: () => this.cancel(),
        })
      ]
    },
  }
})