// Call ngui function to create a ui config.
const FormInstance = window.nguiAsyncImport('public/forms/instance.js')
const FormWorkspace = window.nguiAsyncImport('public/forms/workspace.js')

window.ngui(() => {
  return {
    mixins: [
      window.processManager.mixin()
    ],
    beforeCreate() {
      window.components.setParent(this)
    }, 

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        editWorkspaceUuid: '',
        dialog: {
          remove: false,
          add: false,
          addProject: false,
        },
        apps: [],
        selectable: false,
        loading: false,
      }
    },

    methods: {  
      async init() {
        this.loading = true
        const res = await this.ctx.workspace.fetchWorkspaces()
        this.workspaces = res
        this.loading = false
      },
      
      removeAllSelected() {
        this.removeSelectedWorkspaces().then(() => this.init())
      },

      handleClickMenu(workspace, action) {
        switch (action) {
          case 'edit':
            this.editWorkspaceUuid = workspace.uuid
            this.dialog.add = true
            break
          case 'delete':
            this.selectedWorkspaces = [workspace]
            this.dialog.remove = true
            break
        }
      },

      getAmount(workspace) {
        const nts = workspace.promosnts.length
        const instances = workspace.instances.length
        return nts + instances
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        window.components.page([18], () => this.init()).append(() => [
          window.components.breadcrumbs([
            {
              label: this.ctx.i18n.i18n.t('workspaces'),
              icon: 'fas fa-layer-group',
              tooltip: 'workspaces',
            }
          ]),
          window.components.bar('workspaces').createMenu('file', [
            {
              id: 'addProject',
              label: 'newProject',
              icon: 'add',
              separator: true,
              permissions: this.ctx.globals.getPermissions(['workspacesCreate']),
              click: () => {
                this.dialog.addProject = true
              }
            },
          ]).attrs({
            id: 'workspaces-toolbar',
          }),
          
          window.components.section().append(() => [

            window.components.grid().append(() => [

              window.components.cell(12).append(() => [
                window.components.card('workspaces').attrs({ flat: true }).append(() => [
                  window.components.table(
                    '', 
                    [
                      {
                        i18nLabel: 'name',
                        align: 'left',
                        name: 'name',
                        field: 'name',
                        sortable: true,
                      },
                      {
                        i18nLabel: 'description',
                        align: 'left',
                        name: 'description',
                        field: 'description',
                        sortable: true,
                      },
                      {
                        i18nLabel: 'projects',
                        align: 'left',
                        name: 'projects',
                        field: 'projects',
                        sortable: true,
                      },
                      {
                        i18nLabel: 'action',
                        align: 'left',
                        name: 'action',
                        field: 'action'
                      }
                    ], 
                    this.ctx.workspace.ctx.workspaces,
                  ).attrs({
                    id: 'workspaces-table',
                    selection: 'multiple',
                    selected: this.selectedWorkspaces,
                    rowKey: 'uuid',
                    grid: !this.ctx.platform.browser,
                    flat: false,
                    loading: this.loading,
                    'onUpdate:selected': (val) => this.selectedWorkspaces = val, 
                    'onClick:row': (props) => window.components.router().push('/workspaces/index', { workspace: props.row.uuid }),
                    enableAddBtn: true,
                    enableDeleteBtn: true,
                    permissionsAdd: [17],
                    permissionsDelete: [20],
                    permissionsSelect: [20],
                    'onClick:add': () => {
                      this.dialog.add = true
                    },
                    'onConfirm:delete': () => {
                      this.removeAllSelected()
                    },
                    maxHeight: this.ctx.platform.browser ? '75vh' : undefined,
                  }).append('body-cell-projects', (props) => [
                    window.components.td().append(() => [
                        this.getAmount(props.row)
                    ])
                  ]).append('body-cell-action', (props) => [
                    window.components.td().on({ onClick: this.withModifiers(() => {}, ['stop'])}).append(() => [
                      window.components.defaultMenu('', 'more_vert', {
                        update: [19],
                        delete: [20]
                      }).on({
                        'onClick:menu': (action) => this.handleClickMenu(props.row, action),
                        flat: true,
                        dense: true,
                        round: true,
                      }),
                    ])
                  ]).append('item', (props) => [
                    window.components.tdItem(window.components.card().append(() => [
                      window.components.renderIf(this.selectable, window.components.section().append(() => [
                        window.components.input('select', 'checkbox', props.selected, (val) => props.selected = val).attrs({
                          permissions: this.ctx.globals.getPermissions(['workspacesDelete'])
                        }),
                      ])),
                      window.components.section().append(() => [
                        window.components.item().append('section', () => [
                          window.components.buttonFlatLeft(props.row.name).on({ onClick: () => window.components.router().push('/workspaces/index', { workspace: props.row.uuid })}),
                          window.components.caption(props.row.description)
                        ]).append('side', () => [
                          window.components.element('div').on({ onClick: this.withModifiers(() => {}, ['stop'])}).append(() => [
                            window.components.defaultMenu('', 'more_vert', {
                              update: [19],
                              delete: [20]
                            }).on({
                              'onClick:menu': (action) => this.handleClickMenu(props.row, action),
                              flat: true,
                              dense: true,
                              round: true,
                            }),
                          ])
                        ]),
                        window.components.item('projects', this.getAmount(props.row)),
                      ]),
                    ]))
                  ])
                ]),
              ]),
            ]),

            window.components.dialogDelete(this.selectedWorkspaces.map((item) => item.name).join(', '), this.dialog.remove, (val) => this.dialog.remove = val).on({
              onOk: () => this.removeAllSelected(),
              onCancel: () => this.selectedWorkspaces = [],
            }),  

            window.components.dialog().attrs({
              title: 'workspace',
              modelValue: this.dialog.add,
              'onUpdate:modelValue': (val) => this.dialog.add = val,
            }).append(() => [
              window.components.element(FormWorkspace).attrs({
                workspaceUuid: this.editWorkspaceUuid,
                onCancel: () => this.dialog.add = false,
                onSubmit: () => {
                  this.dialog.add = false
                  this.init()
                },
              })
            ]),
            
            window.components.dialog().attrs({
              title: 'project',
              modelValue: this.dialog.addProject,
              'onUpdate:modelValue': (val) => this.dialog.addProject = val,
            }).append(() => [
              window.components.element(FormInstance).attrs({
                withDMS: true,
                slug: 'local-process-manager',
                instanceUuid: '',
                onCancel: () => this.dialog.addProject = false,
                onSubmit: (instance, form) => {
                  this.dialog.addProject = false
                  this.ctx.workspace.postWorkspace({
                    name: form.name,
                    description: form.description,
                    uuid: this.ctx.uuidv4(),
                    promosnts: [],
                    instances: [
                      {
                        instanceUuid: form.uuid,
                        processManagerUuid: 'local-process-manager',
                        uuid: this.ctx.uuidv4(),
                      }
                    ]
                  })
                  .then(() => {
                    this.init()
                  })
                }
              })
            ]),
          ]),
        ])
      ]
    },
  }
})