import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    
    // Listen on mounted event...
    mounted() {
      // logic
    },

    
    // Listen on unmounted event...
    unmounted() {
      // logic
    },
      
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        workspaceUuid: this.$route.query.workspace,
        dialog: {
          add: false,
          remove: false,
        },
        loading: true,
        apps: [],
        modules: {},
        moduleNamesUsedInWorkspace: [],
        workspace: {},
        instances: [],
      }
    },

    methods: {
      async init() {
        this.loading = true
        this.workspace = await this.ctx.workspace.fetchWorkspace(this.workspaceUuid)
        this.fetchModules()
      },

      fetchInstances() {
        if (!this.workspace.instances) {
          return
        } 
        const promises = this.workspace.instances.map((inst) => this.ctx.processManager.fetchInstanceSettings(inst.processManagerUuid, inst.instanceUuid))
        Promise.all(promises)
          .then((responses) => {
            this.instances = []
            if (responses) {
              responses.forEach((instance) => {
                // If no
                if (!Array.isArray(instance)) {
                  this.instances.push(instance)
                }
              })
            }

            this.filterModules()
          })
          .catch((err) => {
            console.error(err)
          }).finally(() => {
            // processing promos nts
            if (this.workspace.promosnts && this.workspace.promosnts.length && !this.moduleNamesUsedInWorkspace.find((item) => item.id === 'DMS')) {
              this.moduleNamesUsedInWorkspace.unshift(this.modules.processes.find((proc) => proc.id === 'DMS'))
            }
            this.loading = false
          })
      },
      
      fetchModules() {
        this.ctx.httpClient.get('/service/rest/v1/filedb/cfg/modules.json')
          .then((res) => {
            this.modules = res.data.data.data
            this.apps = this.modules.apps
            this.fetchInstances()
          })
          .catch((err) => {
            components.notify().negative(err)
            this.loading = false
          })
      },

      removeWorkspace() {
        this.ctx.workspace.deleteWorkspace(this.workspace.uuid)
          .then(() => {
            this.$router.push('/workspaces')
          })
      },

      filterModules() {
        const modules = {}
        this.instances.forEach((inst) => {
          inst.processes.forEach((proc) => {
            modules[proc.name] = true
          })
        })

        const moduleNames = Object.keys(modules)
        this.moduleNamesUsedInWorkspace = this.modules.processes.filter((proc) => moduleNames.includes(proc.id) || proc.static)
      },

      getBreadcrumb() {
        return processManager.getPMBreadcrumbs(processManager.getPMWorkspaceBreadcrumb())
      }
      
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page([18], () => {
          this.init()
        }).append(() => [
          components.breadcrumbs(this.getBreadcrumb()),
          
          components.bar('workspace').createMenu('file', [
            {
              id: 'edit',
              label: 'edit',
              icon: 'edit',
              permissions: this.ctx.globals.getPermissions(['workspacesUpdate']),
              click: () => {
                this.dialog.add = true
              }
            },
            {
              id: 'delete',
              label: 'delete',
              icon: 'delete',
              permissions: this.ctx.globals.getPermissions(['workspacesDelete']),
              click: () => {
                this.dialog.remove = true
              }
            },
          ]).attrs({
            id: 'workspace-toolbar',
          }),

          window.components.section().append(() => [
            window.components.renderIf(!this.loading && this.moduleNamesUsedInWorkspace.length <= 0, components.bannerInfo('noModulesFound')),
            
            window.components.grid('center', 'center').append(() => [
              ...this.moduleNamesUsedInWorkspace.map((app) => window.components.element('NgSection').attrs({ 
                class: 'col-12 col-md-4 col-lg-2', 
                permissions: app.permissions,
                roles: app.roles || [],
              }).append(() => [
                window.components.button().attrs({
                   fullWidth: true,
                   color: app.color,
                }).append(() => [
                  window.components.column('center', 'center', 'sm').attrs({}).append(() => [
                    window.components.icon(app.icon),
                    window.components.text(app.name).attrs({ class: 'ellipsis' }),
                    window.components.text(app.description).attrs({ class: 'ellipsis' }),          
                  ])
                ]).on({ 
                  onClick: () =>  {
                    if (app.to) {
                      components.router().push(app.to, { workspace: this.workspaceUuid })
                    } else {
                      this.ctx.routing.openURL(app.href)
                    }
                  }
                }).attrs({
                  disable: app.disable
                })
              ])),
            ]),
          ]),
          
          components.dialogDelete(this.workspace.name, this.dialog.remove, (val) => this.dialog.remove = val).on({
            onOk: () => this.removeWorkspace(),
          }), 

          components.formDialog('workspace', this.dialog.add, (val) => this.dialog.add = val, 'public/forms/workspace.js').formOn({
            onSubmit: () => {
              this.dialog.add = false
              this.init()
            },
            onCancel: () => this.dialog.add = false,
          }).formAttrs({
            workspaceUuid: this.workspaceUuid 
          }),

        ])
      ]
    },
  }
})